
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Autodesk.Revit;

namespace RvtMgdDbg {
    public class App : IExternalApplication {

            // get the absolute path of this assembly
        static string ExecutingAssemblyPath = System.Reflection.Assembly.GetExecutingAssembly().Location;
        private AppDocEvents m_appDocEvents;

        public IExternalApplication.Result
        OnStartup(ControlledApplication application)
        {
            //MessageBox.Show("IExternalApplication.OnStartup()");

            AddMenu(application);
            AddToolbar(application);
            AddAppDocEvents(application);

            return IExternalApplication.Result.Succeeded;
        }

        public IExternalApplication.Result
        OnShutdown(ControlledApplication application)
        {
            //MessageBox.Show("IExternalApplication.OnShutdown()");
            RemoveAppDocEvents();

            return IExternalApplication.Result.Succeeded;
        }

        private void
        AddMenu(ControlledApplication app)
        {
            Autodesk.Revit.MenuItem rootMenu = app.CreateTopMenu("RvtMgdDbg");

                // TBD: have to pass in absolute path to the menu item
                // else call backs aren't triggered when documents are changed  (jv - 01/23/07)
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Db..", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopDb");
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Current Selection...", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopModScope");
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Application...", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopApp");
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.SeparatorMenu);
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Test Framework...", ExecutingAssemblyPath, "RvtMgdDbg.CmdTestShell");
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.SeparatorMenu);
            rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Events...", ExecutingAssemblyPath, "RvtMgdDbg.CmdEvents");


                // if you want it to appear as a sub-menu under the "External Tools" menu, use the following
                // code instead.
            /* bool success = rootMenu.AddToExternalTools();
             if (success) {
                 rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Db..", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopDb");
                 rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Current Selection...", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopModScope");
                 rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Snoop Application...", ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopApp");
                 rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.SeparatorMenu);
                 rootMenu.Append(Autodesk.Revit.MenuItem.MenuType.BasicMenu, "Test Framework...", ExecutingAssemblyPath, "RvtMgdDbg.CmdTestShell");
             }
             else
                 System.Windows.Forms.MessageBox.Show("Could not add new menu to External Tools menu!");*/
        }

        private void
        AddToolbar(ControlledApplication app)
        {
            Toolbar toolBar = app.CreateToolbar();
            toolBar.Name = "RvtMgdDbg";
            toolBar.Image = Environment.CurrentDirectory + @"\..\Source\Tools\RvtMgdDbg\res\Toolbar.bmp";  // TBD: this seems limiting!  (you have to have a .bmp floating around with your app, and know the path to it, compiled into your code)

            if (toolBar != null) {
                ToolbarItem item1 = toolBar.AddItem(ExecutingAssemblyPath, "RvtMgdDbg.CmdSnoopDb");
                item1.ToolTip = "Snoop Database";
                item1.ItemType = ToolbarItem.ToolbarItemType.BtnRText;
                item1.ItemText = "Snoop";
                item1.ImageIndex = 0;
                item1.StatusbarTip = "Browse objects in the database";

                ToolbarItem item2 = toolBar.AddItem(ExecutingAssemblyPath, "RvtMgdDbg.CmdTestShell");
                item2.ToolTip = "Test Framework";
                item2.ItemType = ToolbarItem.ToolbarItemType.BtnRText;
                item2.ItemText = "Tests";
                item2.ImageIndex = 1;
                item2.StatusbarTip = "Use a variety of Test/Sample functions";
            }
            else
                System.Windows.Forms.MessageBox.Show("Could not add new toolbar!");
        }

        private void
        AddAppDocEvents(ControlledApplication app)
        {
            m_appDocEvents = new AppDocEvents(app);
            m_appDocEvents.EnableEvents();
        }

        private void
        RemoveAppDocEvents()
        {
            m_appDocEvents.DisableEvents();
        }

    }
}
