//
// (C) Copyright 2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Text;

using Autodesk.Revit;

namespace RvtMgdDbg
{
    public class AppDocEvents
    {
        private ControlledApplication m_app;

        public AppDocEvents(ControlledApplication app)
        {
            m_app = app;
        }

        public void
        EnableEvents()
        {
            m_app.OnDialogBox += new Autodesk.Revit.Events.DialogBoxEventHandler(app_OnDialogBox);
            m_app.OnDocumentClosed += new Autodesk.Revit.Events.DocumentClosedEventHandler(app_OnDocumentClosed);
            m_app.OnDocumentNewed += new Autodesk.Revit.Events.DocumentNewedEventHandler(app_OnDocumentNewed);
            m_app.OnDocumentOpened += new Autodesk.Revit.Events.DocumentOpenedEventHandler(app_OnDocumentOpened);
            m_app.OnDocumentSaved += new Autodesk.Revit.Events.DocumentSavedEventHandler(app_OnDocumentSaved);
            m_app.OnDocumentSavedAs += new Autodesk.Revit.Events.DocumentSavedAsEventHandler(app_OnDocumentSavedAs);
        }

        public void
        DisableEvents()
        {
            m_app.OnDialogBox -= new Autodesk.Revit.Events.DialogBoxEventHandler(app_OnDialogBox);
            m_app.OnDocumentClosed -= new Autodesk.Revit.Events.DocumentClosedEventHandler(app_OnDocumentClosed);
            m_app.OnDocumentNewed -= new Autodesk.Revit.Events.DocumentNewedEventHandler(app_OnDocumentNewed);
            m_app.OnDocumentOpened -= new Autodesk.Revit.Events.DocumentOpenedEventHandler(app_OnDocumentOpened);
            m_app.OnDocumentSaved -= new Autodesk.Revit.Events.DocumentSavedEventHandler(app_OnDocumentSaved);
            m_app.OnDocumentSavedAs -= new Autodesk.Revit.Events.DocumentSavedAsEventHandler(app_OnDocumentSavedAs);
        }

        void app_OnDocumentSavedAs(Document document)
        {
        }

        void app_OnDocumentSaved(Document document)
        {
        }

        void app_OnDocumentOpened(Document document)
        {
            if (EventTrack.Forms.EventsForm.m_docEvents.AreEventsEnabled)
            {
                EventTrack.Forms.EventsForm.m_docEvents.EnableEvents(document);
            }
        }

        void app_OnDocumentNewed(Document document)
        {
            if (EventTrack.Forms.EventsForm.m_docEvents.AreEventsEnabled)
            {
                EventTrack.Forms.EventsForm.m_docEvents.EnableEvents(document);
            }
        }

        void app_OnDocumentClosed(Document document)
        {
            if (EventTrack.Forms.EventsForm.m_docEvents.AreEventsEnabled)
            {
                EventTrack.Forms.EventsForm.m_docEvents.DisableEvents(document);
            }
        }

        void app_OnDialogBox(Autodesk.Revit.Events.DialogBoxData dialogBoxData)
        {
        }
    }
}
