
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Autodesk.Revit;

namespace RvtMgdDbg.EventTrack.Events {

    /// <summary>
    /// Bring up a simple MessageBox to show that we trapped a given event.
    /// </summary>
    
    public class ApplicationEvents : EventsBase {
       
        static public Autodesk.Revit.Application m_app = null;

        public
        ApplicationEvents()
        {
        }

        protected override void
        EnableEventsImp()
        {           
            MessageBox.Show("Application Events Turned On ...", "RvtMgdDbg", MessageBoxButtons.OK, MessageBoxIcon.Information);

            m_app.OnDialogBox += new Autodesk.Revit.Events.DialogBoxEventHandler(m_app_OnDialogBox);
            m_app.OnDocumentClosed += new Autodesk.Revit.Events.DocumentClosedEventHandler(m_app_OnDocumentClosed);
            m_app.OnDocumentNewed += new Autodesk.Revit.Events.DocumentNewedEventHandler(m_app_OnDocumentNewed);
            m_app.OnDocumentOpened += new Autodesk.Revit.Events.DocumentOpenedEventHandler(m_app_OnDocumentOpened);
            m_app.OnDocumentSaved += new Autodesk.Revit.Events.DocumentSavedEventHandler(m_app_OnDocumentSaved);
            m_app.OnDocumentSavedAs += new Autodesk.Revit.Events.DocumentSavedAsEventHandler(m_app_OnDocumentSavedAs);            
        }

        protected override void
        DisableEventsImp ()
        {            
            MessageBox.Show("Application Events Turned Off ...", "RvtMgdDbg", MessageBoxButtons.OK, MessageBoxIcon.Information);

            m_app.OnDialogBox -= new Autodesk.Revit.Events.DialogBoxEventHandler(m_app_OnDialogBox);
            m_app.OnDocumentClosed -= new Autodesk.Revit.Events.DocumentClosedEventHandler(m_app_OnDocumentClosed);
            m_app.OnDocumentNewed -= new Autodesk.Revit.Events.DocumentNewedEventHandler(m_app_OnDocumentNewed);
            m_app.OnDocumentOpened -= new Autodesk.Revit.Events.DocumentOpenedEventHandler(m_app_OnDocumentOpened);
            m_app.OnDocumentSaved -= new Autodesk.Revit.Events.DocumentSavedEventHandler(m_app_OnDocumentSaved);
            m_app.OnDocumentSavedAs -= new Autodesk.Revit.Events.DocumentSavedAsEventHandler(m_app_OnDocumentSavedAs);  
        }       

        private void
        m_app_OnDocumentSavedAs (Document document)
        {
            DisplayEvent("Document SavedAs");
        }

        private void
        m_app_OnDocumentSaved (Document document)
        {
            DisplayEvent("Document Saved");
        }

        private void
        m_app_OnDocumentOpened (Document document)
        {
            DisplayEvent("Document Opened");
        }

        private void
        m_app_OnDocumentNewed (Document document)
        {
            DisplayEvent("Document Newed");
        }

        private void
        m_app_OnDialogBox (Autodesk.Revit.Events.DialogBoxData dialogBoxData)
        {
            DisplayEvent("Dialog Box");
        }

        private void
        m_app_OnDocumentClosed (Document document)
        {
            DisplayEvent("Document Closed");
        }

        private void
        DisplayEvent(string eventStr)
        {
            MessageBox.Show(string.Format("Event: {0}", eventStr), "Application Event", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }       
    }
}
