//
// (C) Copyright 2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Autodesk.Revit;

namespace RvtMgdDbg.EventTrack.Events
{

    /// <summary>
    /// Bring up a simple MessageBox to show that we trapped a given event.
    /// </summary>

    public class DocEvents : EventsBase
    {

        static public Autodesk.Revit.DocumentSet m_docSet = null;

        public
        DocEvents()
        {
        }

        protected override void
        EnableEventsImp()
        {
            MessageBox.Show("Document Events Turned On ...", "RvtMgdDbg", MessageBoxButtons.OK, MessageBoxIcon.Information);

            foreach (Document doc in m_docSet)
            {
                EnableEvents(doc);
            }
        }

        public void
        EnableEvents(Document doc)
        {
            doc.OnClose += new Autodesk.Revit.Events.DocumentCloseEventHandler(doc_OnClose);
            doc.OnSave += new Autodesk.Revit.Events.DocumentSaveEventHandler(doc_OnSave);
            doc.OnSaveAs += new Autodesk.Revit.Events.DocumentSaveAsEventHandler(doc_OnSaveAs);
        }

        protected override void
        DisableEventsImp()
        {
            MessageBox.Show("Document Events Turned Off ...", "RvtMgdDbg", MessageBoxButtons.OK, MessageBoxIcon.Information);

            foreach (Document doc in m_docSet)
            {
                DisableEvents(doc);
            }
        }

        public void
        DisableEvents(Document doc)
        {
            doc.OnClose -= new Autodesk.Revit.Events.DocumentCloseEventHandler(doc_OnClose);
            doc.OnSave -= new Autodesk.Revit.Events.DocumentSaveEventHandler(doc_OnSave);
            doc.OnSaveAs -= new Autodesk.Revit.Events.DocumentSaveAsEventHandler(doc_OnSaveAs);
        }

        void doc_OnSaveAs()
        {
            DisplayEvent("Save as");
        }

        void doc_OnSave()
        {
            DisplayEvent("Save");
        }

        void doc_OnClose()
        {
            DisplayEvent("Close");
        }
        
        private void
        DisplayEvent(string eventStr)
        {
            MessageBox.Show(string.Format("Event: {0}", eventStr), "Document Event", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}
